#version 140

uniform sampler2D texture;
uniform sampler2D specMap;
uniform sampler2D normMap;

in vec3 fragmentLight;
in vec3 fragmentNormal;
in vec3 fragmentEye;

in vec3 test;

in vec3 textureLight;
in vec3 textureEye;

in vec2 fragmentTexCoord;

out vec4 fragColor;

void main(void)
{
	vec3 normal;
	vec3 fragLight;
	vec3 eye;
	float diffuseIntensity;
	float specularIntensity;
	vec3 reflection;

	normal = texture2D(normMap, fragmentTexCoord).xyz * 2.0 - 1.0;
	if((normal.x == -1.0)&&(normal.y == -1.0)&&(normal.z == -1.0))
	{
		normal = normalize(fragmentNormal);
		fragLight = normalize(fragmentLight);
		eye = normalize(fragmentEye);
	}
	else
	{
		normal = normalize(normal);
		//normal.y = -normal.y;
		fragLight = normalize(textureLight);
		eye = normalize(textureEye);
	}

	diffuseIntensity = clamp(max(dot(normal, fragLight), 0.0), 0.0, 1.0);

	float shininess = 10.0;
	reflection = normalize(reflect(-fragLight, normal));
	specularIntensity = pow(clamp(max(dot(reflection, eye), 0.0), 0.0, 1.0), shininess);

	vec4 diffuse = texture2D(texture, fragmentTexCoord);
	vec4 ambient = diffuse*0.005;
	vec4 specMapVal = texture2D(specMap, fragmentTexCoord);
	vec4 specular = vec4(0.2, 0.2, 0.2, 1.0);

	fragColor = ambient + diffuse*diffuseIntensity + specular*specularIntensity*specMapVal;
	fragColor.w = diffuse.w;
	//fragColor = vec4(normalize(test), 1.0);
}
