#version 140
// uniform
uniform mat4 projectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;

uniform vec3 lightDir;

// in
in vec4 vertex;
in vec3 normal;
in vec3 tangent;
in vec2 texCoord;

// out
out vec3 textureLight;
out vec3 textureEye;

out vec3 fragmentLight;
out vec3 fragmentNormal;
out vec3 fragmentEye;

out vec3 test;

out vec2 fragmentTexCoord;

float rand(vec2 seed)
{
	// returns between 0.0 and 1.0
	return fract(sin(dot(seed.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

void main(void)
{
	vec3 tmplight = lightDir;//normalize(lightDir);
	vec3 light = tmplight-(modelMatrix*vertex).xyz;

	// texture
	fragmentTexCoord = texCoord;

	// per pixel lighting
	fragmentLight = normalize((viewMatrix*vec4(light,0.0)).xyz);
	fragmentNormal = normalize((viewMatrix*modelMatrix*vec4(normal,0.0)).xyz);

	fragmentEye = (viewMatrix*modelMatrix*vertex).xyz;
	fragmentEye = -normalize(fragmentEye);

	// tangent
	vec3 n = normalize((viewMatrix*modelMatrix*vec4(normal, 0.0)).xyz);
	vec3 t = normalize((viewMatrix*modelMatrix*vec4(tangent, 0.0)).xyz);
	vec3 b = normalize(cross(n,t));

	textureLight.x = dot(fragmentLight, t);
	textureLight.y = dot(fragmentLight, b);
	textureLight.z = dot(fragmentLight, n);

	textureEye.x = dot(fragmentEye, t);
	textureEye.y = dot(fragmentEye, b);
	textureEye.z = dot(fragmentEye, n);

	test = light;//((viewMatrix*modelMatrix*vertex).xyz+50.0)/100.0;//(t+1.0)/2.0;
	// position
	if(gl_InstanceID==0)
		gl_Position = projectionMatrix*viewMatrix*modelMatrix*vertex;
	else
	{
		float param = rand(vec2(gl_InstanceID*456.789, gl_InstanceID*205.894));
		float param2 = rand(vec2(param*345.345,param*904.347))*2.0-1.0; //-1.0 - 1.0

		// ROTATE
		float rotAngle = gl_InstanceID*0.5;//rand(vec2(param2,param))*6.0;

		mat4 rotateX;
		rotateX[0] = vec4(1.0, 0.0, 0.0, 0.0);
		rotateX[1] = vec4(0.0, cos(rotAngle), sin(rotAngle), 0.0);
		rotateX[2] = vec4(0.0, -sin(rotAngle), cos(rotAngle), 0.0);
		rotateX[3] = vec4(0.0, 0.0, 0.0, 1.0);

		mat4 rotateY;
		rotateY[0] = vec4(cos(rotAngle), 0.0, -sin(rotAngle), 0.0);
		rotateY[1] = vec4(0.0, 1.0, 0.0, 0.0);
		rotateY[2] = vec4(sin(rotAngle), 0.0, cos(rotAngle), 0.0);
		rotateY[3] = vec4(0.0, 0.0, 0.0, 1.0);

		mat4 rotateZ;
		rotateZ[0] = vec4(cos(rotAngle), sin(rotAngle), 0.0, 0.0);
		rotateZ[1] = vec4(-sin(rotAngle), cos(rotAngle), 0.0, 0.0);
		rotateZ[2] = vec4(0.0, 0.0, 1.0, 0.0);
		rotateZ[3] = vec4(0.0, 0.0, 0.0, 1.0);

		mat4 scale;
		float param3 = rand(vec2(param*452.254,param2*248.240))*0.5+0.5;
		float param4 = rand(vec2(param3*538.934,param*892.238))*0.5+0.5;
		float param5 = rand(vec2(param4*843.426,param2*203.458))*0.5+0.5;
		scale[0] = vec4(param3, 0.0, 0.0, 0.0);
		scale[1] = vec4(0.0, param4, 0.0, 0.0);
		scale[2] = vec4(0.0, 0.0, param5, 0.0);
		scale[3] = vec4(0.0, 0.0, 0.0, 1.0);


		mat4 changeMatrix = scale*rotateX*rotateY*rotateZ;

		vec4 tmpVertex = changeMatrix*vertex;
		tmpVertex.x += sin(param*0.8+0.2)*50000.0+param2*2000.0;
		tmpVertex.y += rand(vec2(gl_InstanceID*152.890, gl_InstanceID*255.927))*200.0;
		tmpVertex.z += cos(param*0.8+0.2)*50000.0+param2*2000.0;
		// position
		gl_Position = projectionMatrix*viewMatrix*modelMatrix*tmpVertex;

		// light
		light = tmplight-(modelMatrix*tmpVertex).xyz;
		// per pixel lighting
		fragmentLight = normalize((viewMatrix*vec4(light,0.0)).xyz);
		fragmentNormal = normalize((viewMatrix*modelMatrix*changeMatrix*vec4(normal,0.0)).xyz);

		fragmentEye = (viewMatrix*modelMatrix*tmpVertex).xyz;
		fragmentEye = -normalize(fragmentEye);

		// tangent
		n = normalize((viewMatrix*modelMatrix*changeMatrix*vec4(normal, 0.0)).xyz);
		t = normalize((viewMatrix*modelMatrix*changeMatrix*vec4(tangent, 0.0)).xyz);
		b = normalize(cross(n,t));

		textureLight.x = dot(fragmentLight, t);
		textureLight.y = dot(fragmentLight, b);
		textureLight.z = dot(fragmentLight, n);

		textureEye.x = dot(fragmentEye, t);
		textureEye.y = dot(fragmentEye, b);
		textureEye.z = dot(fragmentEye, n);
	}
}
